@@ -4,9 +4,10 @@ import logging
 import warnings
 from datetime import datetime, timedelta, timezone
 from operator import attrgetter
+from pathlib import Path
 from typing import (
-    TYPE_CHECKING, Any, Callable, Dict, Iterable, Iterator,
-    List, Optional, Set, Tuple, Union, cast, overload
+    TYPE_CHECKING, Any, Callable, Dict, Iterable, Iterator, List,
+    Optional, Set, Tuple, Type, TypeVar, Union, cast, overload
 )
 
 import altair as alt
@@ -41,6 +42,8 @@ if TYPE_CHECKING:
 
 # fmt: on
 
+T = TypeVar("T", bound="Flight")
+
 
 def _tz_interpolate(data, *args, **kwargs):
     return data.astype(int).interpolate(*args, **kwargs).astype(data.dtype)
@@ -104,6 +107,16 @@ class MetaFlight(type):
     def __getattr__(cls, name):
         if name.startswith("aligned_on_"):
             return lambda flight: cls.aligned_on_ils(flight, name[11:])
+        if name.startswith("takeoff_runway_"):
+            return lambda flight: cls.takeoff_from_runway(flight, name[15:])
+        if name.startswith("on_parking_"):
+            return lambda flight: cls.on_parking_position(flight, name[11:])
+        if name.startswith("pushback_"):
+            return lambda flight: cls.pushback(flight, name[9:])
+        if name.startswith("landing_at_"):
+            return lambda flight: cls.landing_at(flight, name[11:])
+        if name.startswith("takeoff_from_"):
+            return lambda flight: cls.takeoff_from(flight, name[13:])
         raise AttributeError
 
 
@@ -135,6 +148,11 @@ class Flight(
         for further processing, e.g. ``groundspeed``, ``vertical_rate``,
         ``track``, ``heading``, ``IAS`` (indicated airspeed) or ``squawk``.
 
+    .. note::
+
+        All navigation related methods are described more in depth on a
+        `dedicated page <navigation.html>`_.
+
     **Abridged contents:**
 
         - properties:
@@ -167,21 +185,14 @@ class Flight(
           `project_shape() <#traffic.core.Flight.project_shape>`_,
           `simplify() <#traffic.core.Flight.simplify>`_,
           `unwrap() <#traffic.core.Flight.unwrap>`_
-        - navigation related method:
-          `closest_point() <#traffic.core.Flight.closest_point>`_,
-          `takeoff_airport() <#traffic.core.Flight.takeoff_airport>`_,
-          `landing_airport() <#traffic.core.Flight.landing_airport>`_,
-          `on_runway() <#traffic.core.Flight.on_runway>`_,
-          `aligned_on_runway() <#traffic.core.Flight.aligned_on_runway>`_,
-          `aligned_on_ils() <#traffic.core.Flight.aligned_on_ils>`_
         - filtering and resampling methods:
           `comet() <#traffic.core.Flight.comet>`_,
           `filter() <#traffic.core.Flight.filter>`_,
           `resample() <#traffic.core.Flight.resample>`_,
         - visualisation with altair:
-          `encode() <#traffic.core.Flight.encode>`_,
+          `chart() <#traffic.core.Flight.chart>`_,
           `geoencode() <#traffic.core.Flight.geoencode>`_
-        - visualisation with leaflet: `layer() <#traffic.core.Flight.layer>`_
+        - visualisation with leaflet: `map_leaflet() <leaflet.html>`_
         - visualisation with Matplotlib:
           `plot() <#traffic.core.Flight.plot>`_,
           `plot_time() <#traffic.core.Flight.plot_time>`_
@@ -289,7 +300,8 @@ class Flight(
     def __getattr__(self, name: str):
         """Helper to facilitate method chaining without lambda.
 
-        TODO improve the documentation
+        Example usage:
+
         flight.altitude_max
             => flight.max('altitude')
         flight.vertical_rate_std
@@ -364,7 +376,8 @@ class Flight(
         return Flight(t.data)  # type: ignore
 
     def next(
-        self, method: Union[str, Callable[["Flight"], Iterator["Flight"]]],
+        self,
+        method: Union[str, Callable[["Flight"], Iterator["Flight"]]],
     ) -> Optional["Flight"]:
         """
         Returns the first segment of trajectory yielded by flight.method()
@@ -512,7 +525,7 @@ class Flight(
     def longer_than(
         self, value: Union[str, timedelta, pd.Timedelta], strict: bool = True
     ) -> bool:
-        """Returns True if flight duration is shorter than value."""
+        """Returns True if flight duration is longer than value."""
         if isinstance(value, str):
             value = pd.Timedelta(value)
         return self.feature_gt(attrgetter("duration"), value, strict)
@@ -947,7 +960,9 @@ class Flight(
 
     @flight_iterator
     def sliding_windows(
-        self, duration: deltalike, step: deltalike,
+        self,
+        duration: deltalike,
+        step: deltalike,
     ) -> Iterator["Flight"]:
 
         duration_ = to_timedelta(duration)
@@ -1031,7 +1046,10 @@ class Flight(
         return getattr(self, name)(*args, **kwargs)(fun)
 
     def apply_time(
-        self, freq: str = "1T", merge: bool = True, **kwargs,
+        self,
+        freq: str = "1T",
+        merge: bool = True,
+        **kwargs,
     ) -> "Flight":
         """Apply features on time windows.
 
@@ -1080,7 +1098,10 @@ class Flight(
         return temp_flight.merge(agg_data, left_on="rounded", right_index=True)
 
     def agg_time(
-        self, freq: str = "1T", merge: bool = True, **kwargs,
+        self,
+        freq: str = "1T",
+        merge: bool = True,
+        **kwargs,
     ) -> "Flight":
         """Aggregate features on time windows.
 
@@ -1334,7 +1355,7 @@ class Flight(
         return self.__class__(data)
 
     def filter_position(self, cascades: int = 2) -> Optional["Flight"]:
-        # TODO improve based on agg_time
+        # TODO improve based on agg_time or EKF
         flight: Optional["Flight"] = self
         for _ in range(cascades):
             if flight is None:
@@ -1415,7 +1436,6 @@ class Flight(
         """
         return self.assign(flight_id=name.format(self=self, idx=idx))
 
-    # TODO change to Iterator
     def onground(self) -> Optional["Flight"]:
         if "altitude" not in self.data.columns:
             return self
@@ -1593,7 +1613,7 @@ class Flight(
         self, other: PointMixin, column_name: str = "bearing"
     ) -> "Flight":
         # temporary, should implement full stuff
-        size = len(self)
+        size = self.data.shape[0]
         return self.assign(
             **{
                 column_name: geo.bearing(
@@ -1676,7 +1696,7 @@ class Flight(
             )
 
         if isinstance(other, PointMixin):
-            size = len(self)
+            size = self.data.shape[0]
             return self.assign(
                 **{
                     column_name: geo.distance(
@@ -1756,7 +1776,7 @@ class Flight(
         **kwargs,
     ) -> "Flight":
 
-        """ Enrich the structure with new ``cumdist`` column computed from
+        """Enrich the structure with new ``cumdist`` column computed from
         latitude and longitude columns.
 
         The first ``cumdist`` value is 0, then distances are computed (in
@@ -1877,24 +1897,10 @@ class Flight(
         # given here for consistency in types
         ...
 
-    def clip(
+    @flight_iterator
+    def clip_iterate(
         self, shape: Union[ShapelyMixin, base.BaseGeometry]
-    ) -> Optional["Flight"]:
-        """Clips the trajectory to a given shape.
-
-        For a shapely Geometry, the first time of entry and the last time of
-        exit are first computed before returning the part of the trajectory
-        between the two timestamps.
-
-        Most of the time, aircraft do not repeatedly come out and in an
-        airspace, but computation errors may sometimes give this impression.
-        As a consequence, the clipped trajectory may have points outside the
-        shape.
-
-        .. warning::
-            Altitudes are not taken into account.
-
-        """
+    ) -> Iterator["Flight"]:
         list_coords = list(self.xy_time)
         if len(list_coords) < 2:
             return None
@@ -1916,7 +1922,10 @@ class Flight(
                 datetime.fromtimestamp(t, timezone.utc)
                 for t in np.stack(intersection.coords)[:, 2]
             )
-            return self.between(min(time_list), max(time_list))
+            between = self.between(min(time_list), max(time_list))
+            if between is not None:
+                yield between
+            return None
 
         def _clip_generator() -> Iterable[Tuple[datetime, datetime]]:
             for segment in intersection:
@@ -1926,15 +1935,42 @@ class Flight(
                 )
                 yield min(times), max(times)
 
-        times: List[Tuple[datetime, datetime]] = list(_clip_generator())
-        clipped_flight = self.between(
-            min(t for t, _ in times), max(t for _, t in times)
-        )
+        for t1, t2 in _clip_generator():
+            between = self.between(t1, t2)
+            if between is not None:
+                yield between
 
-        if clipped_flight is None:
+    def clip(
+        self, shape: Union[ShapelyMixin, base.BaseGeometry]
+    ) -> Optional["Flight"]:
+        """Clips the trajectory to a given shape.
+
+        For a shapely Geometry, the first time of entry and the last time of
+        exit are first computed before returning the part of the trajectory
+        between the two timestamps.
+
+        Most of the time, aircraft do not repeatedly come out and in an
+        airspace, but computation errors may sometimes give this impression.
+        As a consequence, the clipped trajectory may have points outside the
+        shape.
+
+        .. warning::
+            Altitudes are not taken into account.
+
+        """
+
+        t1 = None
+        for segment in self.clip_iterate(shape):
+            if t1 is None:
+                t1 = segment.start
+            t2 = segment.stop
+
+        if t1 is None:
             return None
 
-        if clipped_flight.shape is None:
+        clipped_flight = self.between(t1, t2)
+
+        if clipped_flight is None or clipped_flight.shape is None:
             return None
 
         return clipped_flight
@@ -2150,76 +2186,89 @@ class Flight(
             return ax.plot(*self.shape.xy, **kwargs)
         return []
 
+    def chart(self, *features) -> alt.Chart:  # coverage: ignore
+        """
+        Initializes an altair Chart based on Flight data.
+
+        The features passed in parameters are dispatched to allow plotting
+        multiple features on the same graph.
+
+        Example usage:
+
+        .. code:: python
+
+            # Most simple usage
+            flight.chart().encode(alt.Y("altitude"))
+
+            # With some configuration
+            flight.chart().encode(
+                alt.X(
+                    "utcyearmonthdatehoursminutes(timestamp)",
+                    axis=alt.Axis(title=None, format="%H:%M"),
+                ),
+                alt.Y("altitude", title="altitude (in ft)"),
+                alt.Color("callsign")
+            )
+
+        For a more complex graph plotting similar physical quantities on the
+        same graph, and other quantities on a different graph, the following
+        snippet may be of use.
+
+        .. code:: python
+
+            # More advanced with several plots on the same graph
+            base = (
+                flight.chart("altitude", "groundspeed", "IAS")
+                .encode(
+                    alt.X(
+                        "utcyearmonthdatehoursminutesseconds(timestamp)",
+                        axis=alt.Axis(title=None, format="%H:%M"),
+                    )
+                )
+                .properties(height=200)
+            )
+
+            alt.vconcat(
+                base.transform_filter('datum.variable != "altitude"').encode(
+                    alt.Y(
+                        "value:Q",
+                        axis=alt.Axis(title="speed (in kts)"),
+                        scale=alt.Scale(zero=False),
+                    )
+                ),
+                base.transform_filter('datum.variable == "altitude"').encode(
+                    alt.Y("value:Q", title="altitude (in ft)")
+                ),
+            )
+
+        .. note::
+            See also `plot_time() <#traffic.core.Flight.plot_time>`_ for the
+            Matplotlib equivalent.
+
+        """
+        base = alt.Chart(self.data).encode(
+            alt.X(
+                "utcyearmonthdatehoursminutesseconds(timestamp)",
+                title='alt.X("utcyearmonthdatehoursminutesseconds(timestamp)")',
+            ),
+        )
+        if len(features) > 0:
+            base = base.transform_fold(
+                list(features), as_=["variable", "value"]
+            ).encode(alt.Y("value:Q"), alt.Color("variable:N"))
+
+        return base.mark_line()
+
     def encode(
         self,
         y: Union[str, List[str], alt.Y],
         x: Union[str, alt.X] = "timestamp:T",
         **kwargs,
     ) -> alt.Chart:  # coverage: ignore
-        """Plots the given features according to time.
-
-        The method ensures:
-
-        - only non-NaN data are displayed (no gap in the plot);
-        - the timestamp is naively converted to UTC if not localized.
-
-        Example usage:
-
-        .. code:: python
-
-            flight.encode('altitude')
-            # or with several comparable features
-            flight.encode(['groundspeed', 'IAS', 'TAS'])
-
-        .. warning::
-            No twin axes are available in altair/Vega charts.
-
-        .. note::
-            See also `plot_time() <#traffic.core.Flight.plot_time>`_ for the
-            Matplotlib equivalent.
         """
-        feature_list = ["timestamp"]
-        alt_y: Optional[alt.Y] = None
-        if "flight_id" in self.data.columns:
-            feature_list.append("flight_id")
-        if "callsign" in self.data.columns:
-            feature_list.append("callsign")
-        if "icao24" in self.data.columns:
-            feature_list.append("icao24")
-        if isinstance(y, alt.Y):
-            alt_y = y
-            y = y.shorthand
-        if isinstance(y, str):
-            feature_list.append(y)
-            data = self.data[feature_list].query(f"{y} == {y}")
-            default_encode = dict(
-                x=x,
-                y=alt_y if alt_y is not None else alt.Y(y, title=y),
-                color=alt.Color(
-                    "flight_id"
-                    if "flight_id" in data.columns
-                    else (
-                        "callsign" if "callsign" in data.columns else "icao24"
-                    )
-                ),
-            )
-        else:
-            feature_list += y
-            data = (
-                self.data[feature_list]
-                .melt("timestamp", y)
-                .query("value == value")
-            )
-            default_encode = dict(x="timestamp:T", y="value", color="variable")
-
-        return (
-            alt.Chart(data)
-            .mark_line(interpolate="monotone")
-            .encode(**{**default_encode, **kwargs})
-            .transform_timeunit(
-                timestamp="utcyearmonthdatehoursminutesseconds(timestamp)"
-            )
-        )
+        DEPRECATED: Use Flight.chart() method instead.
+        """
+        raise DeprecationWarning("Use Flight.chart() method instead")
 
     def plot_time(
         self,
@@ -2249,7 +2298,7 @@ class Flight(
             )
 
         .. note::
-            See also `encode() <#traffic.core.Flight.encode>`_ for the altair
+            See also `chart() <#traffic.core.Flight.chart>`_ for the altair
             equivalent.
 
         """
@@ -2285,3 +2334,65 @@ class Flight(
                         ).dt.tz_convert("utc")
                     ).plot(ax=ax, x="timestamp", **kw)
                 )
+
+    @classmethod
+    def from_file(
+        cls: Type[T], filename: Union[Path, str], **kwargs
+    ) -> Optional[T]:
+
+        """Read data from various formats.
+
+        This class method dispatches the loading of data in various format to
+        the proper ``pandas.read_*`` method based on the extension of the
+        filename.
+
+        - .pkl and .pkl.gz dispatch to ``pandas.read_pickle``;
+        - .parquet and .parquet.gz dispatch to ``pandas.read_parquet``;
+        - .json and .json.gz dispatch to ``pandas.read_json``;
+        - .csv and .csv.gz dispatch to ``pandas.read_csv``;
+        - .h5 dispatch to ``pandas.read_hdf``.
+
+        Other extensions return ``None``.
+        Specific arguments may be passed to the underlying ``pandas.read_*``
+        method with the kwargs argument.
+
+        Example usage:
+
+        >>> t = Flight.from_file("example_flight.csv")
+        """
+
+        tentative = super().from_file(filename, **kwargs)
+        if tentative is None:
+            return None
+
+        # Special treatment for flights to download from flightradar24
+        cols_fr24 = {
+            "Altitude",
+            "Callsign",
+            "Direction",
+            "Position",
+            "Speed",
+            "Timestamp",
+            "UTC",
+        }
+        if set(tentative.data.columns) != cols_fr24:
+            return tentative
+
+        latlon = tentative.data.Position.str.split(pat=",", expand=True)
+        return (
+            tentative.assign(
+                latitude=latlon[0].astype(float),
+                longitude=latlon[1].astype(float),
+                timestamp=lambda df: pd.to_datetime(df.UTC),
+            )
+            .rename(
+                columns={
+                    "UTC": "timestamp",
+                    "Altitude": "altitude",
+                    "Callsign": "callsign",
+                    "Speed": "groundspeed",
+                    "Direction": "track",
+                }
+            )
+            .drop(columns=["Timestamp", "Position"])
+        )
